<?php

namespace Drupal\buidlerstribe_customization\Controller;

use Drupal\Core\Link;
use Drupal\node\NodeInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Provides route responses for the buidlerstribe_customization.
 */
class Program extends ControllerBase {

  /**
   * The file storage backend.
   *
   * @var \Drupal\node\NodeStorageInterface
   */
  protected $nodeStorage;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $account;

  /**
   * Constructs a new DrupaliseMe object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The current user.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    AccountInterface $account
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->account = $account;

    $this->nodeStorage = $this->entityTypeManager->getStorage('node');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    // Instantiates this form class.
    return new static(
      $container->get('entity_type.manager'),
      $container->get('current_user'),
    );
  }

  /**
   * Returns a simple page.
   *
   * @return array
   *   A simple renderable array.
   */
  public function apply($id) {
    if (!is_numeric($id)) {
      throw new NotFoundHttpException();
    }

    /** @var \Drupal\node\NodeInterface $program */
    $program = $this->nodeStorage->load($id);
    if (!$program instanceof NodeInterface || ($program->bundle() != 'programs' || !$program->isPublished())) {
      throw new NotFoundHttpException();
    }

    $data = [];
    $data['uid'] = $this->account->id();

    if (!$this->account->id()) {
      $data['message'] = $this->t('Please @login or @create your account first.', [
        '@login' => Link::createFromRoute('login', 'user.login')->toString(),
        '@create' => Link::createFromRoute('register', 'user.register')->toString(),
      ]);
    }
    else {
      $startups = $this->getUserStartups();

      if (empty($startups)) {
        $data['message'] = $this->t('You seem to have not added any startups yet, please add one first from @here.', [
          '@here' => Link::createFromRoute('here', 'node.add', ['node_type' => 'startup_pitch'])->toString(),
        ]);
      }
      elseif (count($startups) == 1) {
        $startup = reset($startups);
        foreach ($startup->get('field_programs')->getValue() as $field) {
          $applied_programs[] = $field['target_id'];
        }
        if (!in_array($id, $applied_programs)) {
          $startup->field_programs->appendItem($id);
        }
        $startup->save();
        $data['message'] = $this->t('Program has been successfully associated with your @startup startup', [
          '@startup' => Link::createFromRoute($startup->getTitle(), 'entity.node.canonical', ['node' => $startup->id()])->toString(),
        ]);
      }
      else {
        $data['message'] = $this->t('Please choose one startup you want to assign this program to');
        foreach ($startups as $startup) {
          $applicable_startups[$startup->id()] = $startup->getTitle();
        }
        $params = [
          'program_id' => $id,
          'applicable_startups' => $applicable_startups,
        ];
        $data['form'] = \Drupal::formBuilder()->getForm('Drupal\buidlerstribe_customization\Form\ProgramSelector', $params);
      }
    }
    return [
      '#theme' => 'apply_programs',
      '#data' => $data,
    ];
  }

  /**
   * Returns a simple page.
   *
   * @return array
   *   A simple renderable array.
   */
  public function applyInvestor($id) {
    if (!is_numeric($id)) {
      throw new NotFoundHttpException();
    }

    /** @var \Drupal\node\NodeInterface $program */
    $program = $this->nodeStorage->load($id);
    if (!$program instanceof NodeInterface || ($program->bundle() != 'investor_profile' || !$program->isPublished())) {
      throw new NotFoundHttpException();
    }

    $data = [];
    $data['uid'] = $this->account->id();

    if (!$this->account->id()) {
      $data['message'] = $this->t('Please @login or @create your account first.', [
        '@login' => Link::createFromRoute('login', 'user.login')->toString(),
        '@create' => Link::createFromRoute('register', 'user.register')->toString(),
      ]);
    }
    else {
      $startups = $this->getUserStartups();

      if (empty($startups)) {
        $data['message'] = $this->t('You seem to have not added any startups yet, please add one first from @here.', [
          '@here' => Link::createFromRoute('here', 'node.add', ['node_type' => 'startup_pitch'])->toString(),
        ]);
      }
      elseif (count($startups) == 1) {
        $startup = reset($startups);
        foreach ($startup->get('field_fund_application')->getValue() as $field) {
          $applied_programs[] = $field['target_id'];
        }
        if (!in_array($id, $applied_programs)) {
          $startup->field_fund_application->appendItem($id);
        }
        $startup->save();
        $data['message'] = $this->t('Investor has been successfully associated with your @startup startup', [
          '@startup' => Link::createFromRoute($startup->getTitle(), 'entity.node.canonical', ['node' => $startup->id()])->toString(),
        ]);
      }
      else {
        $data['message'] = $this->t('Please choose one startup you want to assign this investor to');
        foreach ($startups as $startup) {
          $applicable_startups[$startup->id()] = $startup->getTitle();
        }
        $params = [
          'investor_id' => $id,
          'applicable_startups' => $applicable_startups,
        ];
        $data['form'] = \Drupal::formBuilder()->getForm('Drupal\buidlerstribe_customization\Form\InvestorSelector', $params);
      }
    }
    return [
      '#theme' => 'apply_investor',
      '#data' => $data,
    ];
  }

  /**
   * Returns the startups for the current user.
   *
   * @return array
   *   Returns the startup pitch nodes.
   */
  public function getUserStartups() {
    $nodes = [];
    $query = $this->nodeStorage->getQuery();
    $query->condition('type', 'startup_pitch')
      ->condition('status', 1)
      ->condition('uid', $this->account->id());
    $query = $query->execute();

    if (!empty($query)) {
      $nodes = $this->nodeStorage->loadMultiple($query);
    }

    return $nodes;
  }

  /**
   * Migrate the static programs to dynamic programs.
   */
  public function migrateData() {
    $nodes = [];
    $query = $this->nodeStorage->getQuery();
    $query->condition('type', 'startup_pitch')
      ->condition('status', 1);
    $query = $query->execute();

    if (!empty($query)) {
      $nodes = $this->nodeStorage->loadMultiple($query);
    }

    foreach ($nodes as $startup) {
      if (!$startup->get('field_what_are_you_applying_for_')->isEmpty()) {
        $programs = $startup->get('field_what_are_you_applying_for_')->getValue();
        foreach ($programs as $program) {
          $query = $this->nodeStorage->getQuery();
          $query->condition('type', 'programs')
            ->condition('title', $program['value']);
          $program_query = $query->execute();

          $pid = reset($program_query);
          $program_ref = $startup->get('field_programs')->getValue();

          $f_programs = [];
          foreach ($program_ref as $applied_programs) {
            $f_programs[] = $applied_programs['target_id'];
          }

          if (!in_array($pid, array_unique($f_programs))) {
            $startup->field_programs[] = ['target_id' => $pid];
          }
        }
        $startup->save();
      }
    }

    exit;
  }

  /**
   * Migrate the static programs to dynamic programs.
   */
  public function updateStatFields() {
    $nodes = [];
    $query = $this->nodeStorage->getQuery();
    $query->condition('type', 'startup_pitch')
      ->condition('status', 1);
    $query = $query->execute();

    if (!empty($query)) {
      $nodes = $this->nodeStorage->loadMultiple($query);
    }

    foreach ($nodes as $update) {
      $buidler_service = \Drupal::service('buidlerstribe_customization.buidlerstribeservice');
      $buidler_service->updateStartupStats($update->id());
      $buidler_service->updateVouchesStats($update->id());
      $buidler_service->updateQuestionStats($update->id());
      $buidler_service->updatefundraisingStats($update->id());

    }

    exit;
  }

}
