<?php

namespace Drupal\mail_events\EventSubscriber;

use Psr\Log\LoggerInterface;
use Drupal\flag\FlagServiceInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\mail_events\Services\Mailer;
use Drupal\mail_events\Event\NodeEvents;
use Drupal\mail_events\Event\NodeInsert;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\mail_events\Event\InsertQuestionReply;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Logs the creation of a new node.
 */
class NodeEventSubscriber implements EventSubscriberInterface {

  /**
   * The entity type targeted by this resource.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The flag service.
   *
   * @var \Drupal\flag\FlagServiceInterface
   */
  protected $flagService;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * InitSubscriber constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   EntityTypeManagerInterface to load entity.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Config factory.
   * @param \Drupal\flag\FlagServiceInterface $flag_service
   *   The flag service.
   * @param \Drupal\mail_events\Services\Mailer $mail_manager
   *   The mail manager.
   * @param \Psr\Log\LoggerInterface $logger
   *   Logger.
   */
  public function __construct(
    EntityTypeManagerInterface $entityTypeManager,
    ConfigFactoryInterface $config_factory,
    FlagServiceInterface $flag_service,
    Mailer $mail_manager,
    LoggerInterface $logger
  ) {
    $this->entityTypeManager = $entityTypeManager;
    $this->configFactory = $config_factory;
    $this->flagService = $flag_service;
    $this->mailManager = $mail_manager;
    $this->logger = $logger;
  }

  /**
   * Log the creation of a reply on a comment on questions.
   *
   * @param \Drupal\event_subscriber_demo\Event\NodeInsert $event
   *   Wraps a node insertion demo event for event listeners.
   */
  public function onNodeInsert(NodeInsert $event) {
    $entity = $event->getEntity();
    $bundle = $entity->bundle();

    if ($bundle == 'startup_pitch') {
      $this->onInsertStartupPitch($entity);
    }

    if ($bundle == 'startup_updates') {
      $this->onInsertStartupUpdate($entity);
    }

    if ($bundle == 'vouches') {
      $this->onInsertVouchUpdate($entity);
    }

    if ($bundle == 'questions') {
      $this->onInsertQuestionUpdate($entity);
    }
  }

  /**
   * Log the creation of a reply on a comment on questions.
   *
   * @param \Drupal\event_subscriber_demo\Event\InsertQuestionReply $event
   *   Wraps a node insertion demo event for event listeners.
   */
  public function onInsertQuestionReply(InsertQuestionReply $event) {
    $entity = $event->getEntity()->getCommentedEntity();
    /** @var \Drupal\user\UserInterface $author */
    $author = $this->entityTypeManager->getStorage('user')->load($entity->getOwnerid());
    $referenced_profile_id = $entity->field_referenced_profile->target_id;
    $referenced_profile = $this->entityTypeManager->getStorage('node')->load($referenced_profile_id);
    $replier = $event->getEntity()->getOwner();

    if ($referenced_profile->getOwnerId() != $replier->id()) {
      $name = $replier->field_name->value;
      // Log a notification.
      $notificationService = \Drupal::service('notifications.logger');
      $message = [
        'id' => $entity->id(),
        'bundle' => $entity->bundle(),
        'title' => $name . ' answered your question',
        'content' => $event->getEntity()->field_comment->value,
      ];
      $notificationService->logNotification($message, 'buidlertribe', $referenced_profile);
    }

    // Get the emailer content.
    if ($entity->getOwnerid() != $event->getEntity()->getOwnerid()) {
      $config = $this->configFactory->get('mail_events.settings');
      $subject = $config->get('question_reply_subject');
      $subject = str_replace('@founder', $event->getEntity()->getAuthorName(), $subject);
      $subject = str_replace('@startup', $referenced_profile->title->value, $subject);

      $body = $config->get('question_reply_body');
      $body = str_replace('@founder', $event->getEntity()->getAuthorName(), $body);
      $body = str_replace('@startup', $referenced_profile->title->value, $body);
      $body = str_replace('@startup_link', $referenced_profile->toLink()->toString(), $body);

      // Prepare to send mail.
      $to = $author->getEmail();
      $params['title'] = $referenced_profile->title->value;
      $params['subject'] = $subject;
      $params['message'] = $body;
      $this->mailManager->sendEmail('mail_events', 'question_reply', $to, 'en', $params);
    }
  }

  /**
   * Log the creation of a new node.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   Wraps a node insertion demo event for event listeners.
   */
  public function onInsertStartupPitch(EntityInterface $entity) {
    /** @var \Drupal\user\UserInterface $author */
    $author = $this->entityTypeManager->getStorage('user')->load($entity->getOwnerId());

    $config = $this->configFactory->get('mail_events.settings');
    $subject = $config->get('drip_subject');
    $body = $config->get('drip_body');

    $to = $author->getEmail();
    $params['message'] = $body;
    $params['subject'] = $subject;
    $this->mailManager->sendEmail('mail_events', 'drip_email', $to, 'en', $params);
  }

  /**
   * Log the creation of a new node.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   Wraps a node insertion demo event for event listeners.
   */
  public function onInsertStartupUpdate(EntityInterface $entity) {
    $referenced_profile_id = $entity->field_referenced_profile->target_id;
    $referenced_profile = $this->entityTypeManager->getStorage('node')->load($referenced_profile_id);
    $users = $this->flagService->getFlaggingUsers($referenced_profile);
    $following_users = [];

    // Log a notification.
    $notificationService = \Drupal::service('notifications.logger');
    $message = [
      'id' => $entity->id(),
      'bundle' => $entity->bundle(),
      'title' => $referenced_profile->title->value . ' posted an update',
      'content' => $entity->field_body->value,
    ];
    $notificationService->logNotification($message, 'buidlertribe', $referenced_profile);

    if ($users) {
      foreach ($users as $user) {
        $following_users[] = $user->getEmail();
      }
      // Get the emailer content.
      if ($entity->field_body->value) {
        $config = $this->configFactory->get('mail_events.settings');
        $subject = $config->get('startup_updates_subject');
        $subject = str_replace('@title', $referenced_profile->title->value, $subject);

        $body = $config->get('startup_updates_body');
        $body = str_replace('@startup_profile', $referenced_profile->title->value, $body);
        $body = str_replace('@update_title', $entity->field_body->value, $body);
        $body = str_replace('@startup_link', $referenced_profile->toLink()->toString(), $body);

        // Prepare to send mail.
        $to = implode(',', $following_users);
        $params['title'] = $referenced_profile->title->value;
        $params['message'] = $body;
        $params['subject'] = $subject;
        $this->mailManager->sendEmail('mail_events', 'new_update', $to, 'en', $params);
      }
    }
  }

  /**
   * Log the creation of a new vouch.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   Wraps a node insertion demo event for event listeners.
   */
  public function onInsertVouchUpdate(EntityInterface $entity) {
    $referenced_profile_id = $entity->field_referenced_profile->target_id;
    $referenced_profile = $this->entityTypeManager->getStorage('node')->load($referenced_profile_id);
    /** @var \Drupal\user\UserInterface $author */
    $author = $this->entityTypeManager->getStorage('user')->load($referenced_profile->getOwnerId());

    // Log a notification.
    $notificationService = \Drupal::service('notifications.logger');
    $message = [
      'id' => $entity->id(),
      'bundle' => $entity->bundle(),
      'title' => $referenced_profile->title->value . ' has a new vouch',
      'content' => $entity->field_body->value,
    ];
    $notificationService->logNotification($message, 'buidlertribe', $referenced_profile);

    // Get the emailer content.
    if ($entity->field_body->value) {
      $config = $this->configFactory->get('mail_events.settings');
      $subject = $config->get('vouch_create_subject');
      $subject = str_replace('@title', $referenced_profile->title->value, $subject);

      $body = $config->get('vouch_create_body');
      $body = str_replace('@startup_profile', $referenced_profile->title->value, $body);
      $body = str_replace('@investor_name', $author->getEmail(), $body);
      $body = str_replace('@startup_link', $referenced_profile->toLink()->toString(), $body);

      // Prepare to send mail.
      $to = $author->getEmail();
      $params['title'] = $referenced_profile->title->value;
      $params['subject'] = $subject;
      $params['message'] = $body;
      $this->mailManager->sendEmail('mail_events', 'vouch_create', $to, 'en', $params);
    }
  }

  /**
   * Log the creation of a new vouch.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   Wraps a node insertion demo event for event listeners.
   */
  public function onInsertQuestionUpdate(EntityInterface $entity) {
    $referenced_profile_id = $entity->field_referenced_profile->target_id;
    $referenced_profile = $this->entityTypeManager->getStorage('node')->load($referenced_profile_id);
    /** @var \Drupal\user\UserInterface $author */
    $author = $this->entityTypeManager->getStorage('user')->load($referenced_profile->getOwnerId());

    // Log a notification.
    $notificationService = \Drupal::service('notifications.logger');
    $message = [
      'id' => $entity->id(),
      'bundle' => $entity->bundle(),
      'title' => $referenced_profile->title->value . ' has a new question',
      'content' => $entity->field_body->value,
    ];
    $notificationService->logNotification($message, 'buidlertribe', $referenced_profile);

    // Get the emailer content.
    if ($entity->field_body->value) {
      $config = $this->configFactory->get('mail_events.settings');
      $subject = $config->get('question_create_subject');
      $subject = str_replace('@title', $referenced_profile->title->value, $subject);

      $body = $config->get('question_create_body');
      $body = str_replace('@startup_profile', $referenced_profile->title->value, $body);
      $body = str_replace('@investor_name', $author->getEmail(), $body);
      $body = str_replace('@startup_link', $referenced_profile->toLink()->toString(), $body);

      // Prepare to send mail.
      $to = $author->getEmail();
      $params['title'] = $referenced_profile->title->value;
      $params['subject'] = $subject;
      $params['message'] = $body;
      $this->mailManager->sendEmail('mail_events', 'question_create', $to, 'en', $params);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[NodeEvents::NODE_INSERT][] = ['onNodeInsert'];
    $events[NodeEvents::COMMENT_INSERT][] = ['onInsertQuestionReply'];
    return $events;
  }

}
