<?php

namespace Drupal\mail_events\Services;

use Psr\Log\LoggerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Service for to return applicable drip contents.
 *
 * @package Drupal\mail_events
 */
class Drip {

  /**
   * A logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The node storage manager.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $nodeStorage;

  /**
   * InitSubscriber constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   EntityTypeManagerInterface to load entity.
   * @param \Drupal\mail_events\Services\Mailer $mail_manager
   *   The mail manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Config factory.
   * @param \Psr\Log\LoggerInterface $logger
   *   Logger object.
   */
  public function __construct(
    EntityTypeManagerInterface $entityTypeManager,
    Mailer $mail_manager,
    ConfigFactoryInterface $config_factory,
    LoggerInterface $logger
  ) {
    $this->entityTypeManager = $entityTypeManager;
    $this->mailManager = $mail_manager;
    $this->configFactory = $config_factory;
    $this->logger = $logger;

    $this->nodeStorage = $this->entityTypeManager->getStorage('node');
  }

  /**
   * Returns the node aged as per the param passed.
   *
   * @param string $bundle
   *   Machine name of the content type.
   *
   * @return array
   *   Returns the node title and author email in array.
   */
  public function getDrips($bundle) {
    $config = $this->configFactory->get('mail_events.settings');
    $interval_days = [1, 2];

    $drip_email_detail = [];
    $now = time();
    $query = $this->nodeStorage->getQuery();
    $query->condition('type', $bundle)
      ->condition('status', 1);
    $ids = $query->execute();
    $nodes = $this->nodeStorage->loadMultiple($ids);

    /** @var \Drupal\node\NodeInterface $node */
    foreach ($nodes as $node) {
      // Get the interval between now and node creation date in days.
      $interval = round(($now - $node->getCreatedTime()) / (60 * 60 * 24));
      if (in_array($interval, $interval_days)) {
        $drip_email_detail[$node->id()] = [
          'interval' => $interval,
          'title' => $node->getTitle(),
          'author' => $node->getOwner()->getEmail(),
        ];
      }
    }

    if (!empty($drip_email_detail)) {
      foreach ($drip_email_detail as $detail) {
        $this->sendDripMail($detail);
      }
    }
    return TRUE;
  }

  /**
   * Drip service.
   */
  public function sendDripMail($detail) {
    $config = $this->configFactory->get('mail_events.settings');
    $subject = $config->get('drip_subject_' . $detail['interval']);
    $subject = str_replace('@days', $detail['interval'], $subject);
    $body = $config->get('drip_body_' . $detail['interval']);

    // Prepare to send mail.
    $to = $detail['author'];
    $params['subject'] = $subject;
    $params['message'] = $body;
    $this->mailManager->sendEmail('mail_events', 'drip_email', $to, 'en', $params);
  }

}
